/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */

/* Copyright (c) Chris Vine, 2011 and 2012
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

const St = imports.gi.St;
const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const AppFavorites = imports.ui.appFavorites;
const ExtensionSystem = imports.ui.extensionSystem;

const Mainloop = imports.mainloop;
const Gettext = imports.gettext;


var button = null;
var button_pos = 1;
var id = null;
var use_icon = false;
var menu_actor = null;

function set_panel_display(button) {
    if (menu_actor) button.actor.remove_actor(menu_actor);

    if (use_icon) {
        let fpath = Gio.file_new_for_path(ExtensionSystem.extensionMeta["favorites@cvine.org"].path +
                                          "/emblem-favorite.png");
        let icon = new St.Icon({gicon: new Gio.FileIcon({file: fpath}),
                                icon_type: St.IconType.SYMBOLIC,
                                style_class: "favorites-icon"});
        button.actor.add_actor(icon);
        menu_actor = icon;
    }
    else {
        let label = new St.Label({text: Gettext.gettext("Favorites")});
        button.actor.add_actor(label);
        menu_actor = label;
    }
}

function add_item(button, app) {
    let item = new PopupMenu.PopupBaseMenuItem;
    button.menu.addMenuItem(item);
    let box = new St.BoxLayout({vertical: false,
				pack_start: false,
				style_class: "favorites-menu-box"});
    item.addActor(box);
    let icon = app.create_icon_texture(24);
    box.add(icon);
    let label = new St.Label({text: app.get_name()});
    box.add(label);
    item.connect("activate", function () {app.open_new_window(-1);});
}

function add_prefs(button) {
    let sub = new PopupMenu.PopupSubMenuMenuItem(Gettext.gettext("Preferences"));
    button.menu.addMenuItem(sub);
    let item = new PopupMenu.PopupSwitchMenuItem(Gettext.gettext("Panel Icon"),
                                                 use_icon,
                                                 {style_class: "favorites-preferences"});
    item.connect("activate", function (item, event) {
        use_icon = item.state;
        save_setting();
        set_panel_display(button);
    });
    sub.menu.addMenuItem(item);

    let combo = new PopupMenu.PopupComboBoxMenuItem({style_class: "favorites-preferences"});
    combo.connect("active-item-changed", function (item, position) {
        if (button && button_pos !== position) {
            Main.panel._leftBox.remove_actor(button.actor);
            Main.panel._leftBox.insert_actor(button.actor, position);
            Main.panel._leftBox.child_set(button.actor, {y_fill: true});
            button_pos = position;
            save_setting();
        }
    });
    sub.menu.addMenuItem(combo);
    for (let count = 0; count < 5;) {
        let item = new PopupMenu.PopupMenuItem(Gettext.gettext("Panel Position") + ": " + (++count),
                                               {style_class: "favorites-preferences"});
        combo.addMenuItem(item);
    }
    combo.setActiveItem(button_pos);

    let separator = new PopupMenu.PopupSeparatorMenuItem;
    button.menu.addMenuItem(separator);
}

function build_menu(button) {
    add_prefs(button);
    AppFavorites.getAppFavorites().getFavorites().forEach(function (app) {
        add_item(button, app);
    });
};

// get_setting() and save_setting() are a temporary arrangement until
// we can use GSettings in gnome-shell-3.4
function get_setting() {
    let config_file = ExtensionSystem.extensionMeta["favorites@cvine.org"].path + "/setting";
    if (GLib.file_test(config_file, GLib.FileTest.EXISTS)) {
        let text = GLib.file_get_contents(config_file)[1].toString();
        use_icon = text.search(/icon/) != -1;
        let pos = text.search(/:\d/);
        if (pos === -1) button_pos = 1;
        else button_pos = Number(text[pos + 1]);
    }
    else {
        GLib.file_set_contents(config_file, "text:1", -1);
        use_icon = false;
        button_pos = 1;
    }
}

function save_setting() {
    let config_file = ExtensionSystem.extensionMeta["favorites@cvine.org"].path + "/setting";
    let setting = (use_icon) ? "icon:" + button_pos
                             : "text:" + button_pos;
    GLib.file_set_contents(config_file, setting, -1);
}

function enable() {
    button = new PanelMenu.Button(0.0);
    get_setting();
    set_panel_display(button);
    build_menu(button);

    id = global.settings.connect("changed::" + AppFavorites.getAppFavorites().FAVORITE_APPS_KEY,
				 function() {
				     button.menu.removeAll();
				     build_menu(button);
				 });

    // A one-shot timer of 100mS duration so that our panel button is
    // the last to be inserted in the panel's left box.  This is
    // necessary to ensure that the user's chosen position is
    // respected.  This does not give rise to any race condition with
    // respect to the user's actions on the GUI, because until the
    // timeout callback executes the favorites menu cannot be acted
    // on by the user (it is not in the panel).
    Mainloop.timeout_add(100, function () {
        Main.panel._leftBox.insert_actor(button.actor, button_pos);
        Main.panel._leftBox.child_set(button.actor, {y_fill: true});
        Main.panel._menus.addMenu(button.menu);
        return false;
    });
}

function disable() {
    if (id) global.settings.disconnect(id);
    if (button) button.destroy();
    button = null;
    id = null;
    menu_actor = null;
}

function init() {
    let user_locale_path = ExtensionSystem.extensionMeta["favorites@cvine.org"].path + "/locale";
    Gettext.bindtextdomain("favorites", user_locale_path);
    Gettext.textdomain("favorites");
}
